#   DrCr: Web-based double-entry bookkeeping framework
#   Copyright (C) 2022–2023  Lee Yingtong Li (RunasSudo)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from drcr import AMOUNT_DPS
from drcr.database import db
from drcr.models import AccountConfiguration, Amount, Metadata, Transaction, TrialBalancer
from drcr.reports import Calculated, Report, Section, Spacer, Subtotal, entries_for_kind

from .tax_tables import base_tax, repayment_rates, fbt_grossup

from datetime import datetime

def eofy_date():
	"""Get the datetime for the end of the financial year"""
	
	return datetime.strptime(Metadata.get('eofy_date'), '%Y-%m-%d')

def base_income_tax(year, taxable_income):
	"""Get the amount of base income tax"""
	
	for i, (upper_limit, flat_amount, marginal_rate) in enumerate(base_tax[year]):
		if upper_limit is None or taxable_income.quantity <= upper_limit * (10**AMOUNT_DPS):
			lower_limit = base_tax[year][i - 1][0]
			return Amount(flat_amount * (10**AMOUNT_DPS) + marginal_rate * (taxable_income.quantity - lower_limit * (10**AMOUNT_DPS)), '$')

def lito(taxable_income):
	"""Get the amount of low income tax offset"""
	
	if taxable_income.quantity <= 3750000:
		return Amount(70000, '$')
	if taxable_income.quantity <= 4500000:
		return Amount(70000 - 0.05 * (taxable_income.quantity - 3750000), '$')
	if taxable_income.quantity <= 6666700:
		return Amount(32500 - int(0.015 * (taxable_income.quantity - 4500000)), '$')
	
	return Amount(0, '$')

def medicare_levy(taxable_income):
	if taxable_income.quantity < 2920700:
		raise NotImplementedError('Medicare levy reduction is not implemented')
	
	return Amount(int(taxable_income.quantity * 0.02), '$')

def study_loan_repayment(year, taxable_income, rfb_grossedup):
	"""Get the amount of mandatory study loan repayment"""
	
	repayment_income = taxable_income + rfb_grossedup
	
	for upper_limit, rate in repayment_rates[year]:
		if upper_limit is None or repayment_income.quantity <= upper_limit * (10**AMOUNT_DPS):
			return Amount(rate * repayment_income.quantity, '$')

def tax_summary_report():
	# Get trial balance
	balancer = TrialBalancer()
	balancer.apply_transactions(db.session.scalars(db.select(Transaction).where(Transaction.dt <= eofy_date()).options(db.selectinload(Transaction.postings))).all())
	
	accounts = dict(sorted(balancer.accounts.items()))
	
	# Get account configurations
	account_configurations = AccountConfiguration.get_all_kinds()
	
	report = Report(title='Tax summary')
	report.entries = [
		Section(
			entries=[
				Section(
					title='Salary or wages (1)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.income1', neg=True, floor=100) + [Subtotal('Total item 1', id='income1')]
				),
				Spacer(),
				Section(
					title='Australian Government allowances and payments (5)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.income5', neg=True) + [Subtotal('Total item 5', id='income5', floor=100)]
				),
				Spacer(),
				Section(
					title='Gross interest (10)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.income10', neg=True) + [Subtotal('Total item 10', id='income10', floor=100)]
				),
				Spacer(),
				Section(
					title='Partnerships and trusts (13)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.income13', neg=True, floor=100) + [Subtotal('Total item 13', id='income13', floor=100)]
				),
				Spacer(),
				#Section(
				#	title='Net capital gains (18)',
				#	entries=entries_for_kind(account_configurations, accounts, 'austax.income18', neg=True) + [Subtotal('Total item 18', id='income18', floor=100)]
				#),
				#Spacer(),
				Section(
					title='Foreign source income and foreign assets or property (20)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.income20', neg=True, floor=100) + [Subtotal('Total item 20', id='income20', floor=100)]
				),
				Spacer(),
				Calculated(
					'Total assessable income',
					lambda r: r.by_id('income1').amount + r.by_id('income5').amount + r.by_id('income10').amount + r.by_id('income13').amount + r.by_id('income20').amount,
					id='assessable',
					heading=True,
					bordered=True
				),
				Spacer(),
				Section(
					title='Work-related travel expenses (D2)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.d2') + [Subtotal('Total item D2', id='d2', floor=100)]
				),
				Spacer(),
				Section(
					title='Work-related self-education expenses (D4)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.d4') + [Subtotal('Total item D4', id='d4', floor=100)]
				),
				Spacer(),
				Section(
					title='Other work-related expenses (D5)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.d5') + [Subtotal('Total item D5', id='d5', floor=100)]
				),
				Spacer(),
				Section(
					title='Gifts or donations (D9)',
					entries=entries_for_kind(account_configurations, accounts, 'austax.d9') + [Subtotal('Total item D9', id='d9', floor=100)]
				),
				Spacer(),
				Calculated(
					'Total deductions',
					lambda r: r.by_id('d2').amount + r.by_id('d4').amount + r.by_id('d5').amount + r.by_id('d9').amount,
					id='deductions',
					heading=True,
					bordered=True
				),
				Spacer(),
				Calculated(
					'Net taxable income',
					lambda r: r.by_id('assessable').amount - r.by_id('deductions').amount,
					id='taxable',
					heading=True,
					bordered=True
				),
				Spacer(),
				Section(
					entries=[
						Calculated(
							'Base income tax',
							lambda _: base_income_tax(eofy_date().year, report.by_id('taxable').amount)
						),
						Calculated(
							'Medicare levy',
							lambda _: medicare_levy(report.by_id('taxable').amount)
						),
						Subtotal('Total income tax', id='total_tax', bordered=True)
					]
				),
				Spacer(),
				Section(
					title='Tax offsets',
					entries=entries_for_kind(account_configurations, accounts, 'austax.offset', neg=True) + [
						Calculated('Low income tax offset', lambda _: lito(report.by_id('taxable').amount)),
						Subtotal('Total tax offsets', id='offsets')
					]
				),
				Spacer(),
				Section(
					entries=[
						Calculated(
							'Taxable value of reportable fringe benefits',
							lambda _: -sum((e.amount for e in entries_for_kind(account_configurations, accounts, 'austax.rfb')), Amount(0, '$')),
							id='rfb_taxable'
						),
						Calculated(
							'Grossed-up value',
							lambda _: Amount(report.by_id('rfb_taxable').amount.quantity * fbt_grossup[eofy_date().year], '$'),
							id='rfb_grossedup'
						),
						Calculated(
							'Mandatory study loan repayment',
							lambda _: study_loan_repayment(eofy_date().year, report.by_id('taxable').amount, report.by_id('rfb_grossedup').amount),
							id='loan_repayment',
							heading=True
						)
					]
				),
				Spacer(),
				Section(
					title='PAYG withheld amounts',
					entries=entries_for_kind(account_configurations, accounts, 'austax.paygw') + [Subtotal('Total withheld amounts', id='paygw')]
				),
				Spacer(),
				Calculated(
					'ATO liability payable (refundable)',
					lambda _: report.by_id('total_tax').amount - report.by_id('offsets').amount - report.by_id('paygw').amount + report.by_id('loan_repayment').amount,
					heading=True,
					bordered=True
				)
			]
		)
	]
	report.calculate()
	
	return report
